#!/usr/bin/perl -w -I /daten/netsaint/libexec
#
# check_cisco_cpu.pl - netsaint plugin 
# 
#
#
#
# Copyright (C) 2000 Christoph Kron
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# IMPORT INFORMATION
# this module requires the perl module Net::SNMP
# the latest version of Net::SNMP can be found on:
# http://search.cpan.org/search?module=Net::SNMP
# or at http://www.cpan.org
#
#
# Report bugs to: ck@zet.net
#
# 09.06.2000 Version 1.0

use strict;

use Net::SNMP;
use Getopt::Long;

&Getopt::Long::config('auto_abbrev');


my $status;
my $TIMEOUT = 15;

my %ERRORS = ('UNKNOWN' , '-1',
              'OK' , '0',
              'WARNING', '1',
              'CRITICAL', '2');


my $state = "UNKNOWN";
my $answer = "";
my $snmpkey = 1;
my $community = "public";
my $port = 161;
my $cisco = 0;
my @snmpoids;
my $snmpBusyPer;
my $snmpAvgBusy5;
my $snmpcpu;
my $hostname;
my $session;
my $error;
my $response;
my $cpu = 0;
my $cpu1 = 0;
my $cpu5 = 0;
my $cpuwarn;
my $cpucrit;

sub usage {
  printf "\nMissing arguments!\n";
  printf "\n";
  printf "Perl Check ciscocpu plugin for NetSaint\n";
  printf  "monitors cpu performance of Cisco routers\n";
  printf "usage: \n";
  printf "check_ciscocpu.pl [-d]  -c <READCOMMUNITY> [-p <PORT>] <HOSTNAME>";
  printf " [WARN] [CRIT]";
  printf "\nCopyright (C) 2000 Christoph Kron\n";
  printf "check_ciscocpu.pl comes with ABSOLUTELY NO WARRANTY\n";
  printf "This programm is licensed under the terms of the ";
  printf "GNU General Public License\n(check source code for details)\n";
  printf "\n\n";
  exit $ERRORS{"UNKNOWN"};
}

# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub {
     print ("ERROR: No snmp response from $hostname (alarm)\n");
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);

my $hrefstart;
my $hrefend;

$status = GetOptions( "community=s",\$community,
                     "port=i",\$port);
if ($status == 0)
{
        &usage;
}
  
   #shift;
   $hostname  = shift || &usage;
   $cpuwarn = shift || 75;
   $cpucrit = shift || 90;

   ($session, $error) = Net::SNMP->session(
      -hostname  => $hostname,
      -community => $community,
      -port      => $port
   );

   if (!defined($session)) {
      $state='UNKNOWN';
      $answer=$error;
      print ("$state: $answer");
      exit $ERRORS{$state};
   }

   $snmpBusyPer='1.3.6.1.4.1.9.2.1.56.0';
   $snmpAvgBusy5='1.3.6.1.4.1.9.2.1.58.0';


   push(@snmpoids,$snmpBusyPer);
   push(@snmpoids,$snmpAvgBusy5);

   if (!defined($response = $session->get_request(@snmpoids))) {
      $answer=$session->error;
      $session->close;
      $state = 'UNKNOWN';
      print ("$answer,$community,$snmpkey");
      exit $ERRORS{$state};
   }

   $cpu = $response->{$snmpAvgBusy5};
   $cpu1 = $response->{$snmpBusyPer};
   $cpu5 = $cpu;

   if ($cpu > $cpucrit) {
      $state='CRITICAL';
   }
   elsif ($cpu > $cpuwarn) {
      $state='WARNING';
   }
   else {
      $state = 'OK';
   }
        $hrefstart = "";
        $hrefend = "";
   $answer = sprintf("%s$state ,cpu: %1.1f, 5-min cpu: %1.1f%s \n",
		      $hrefstart,
                      $cpu1,
                      $cpu5,
		      $hrefend);

   $session->close;

print ("$answer");
exit $ERRORS{$state};

