#!/usr/bin/perl -w -I /daten/netsaint/libexec
#
# check_filervolume.pl - netsaint plugin 
# 
#
#
#
# Copyright (C) 2000 Christoph Kron
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# IMPORT INFORMATION
# this module requires the perl module Net::SNMP
# the latest version of Net::SNMP can be found on:
# http://search.cpan.org/search?module=Net::SNMP
# or at http://www.cpan.org
#
# this plugin produces long output
# in base/netsaint.h increase #define MAX_INPUT_BUFFER to 1024
# then recompile and install netsaint for the changes to take effect
#
#
# Report bugs to: ck@zet.net
#
# 07.10.2000 Version 0.8

use strict;

use Net::SNMP;
use Getopt::Long;

&Getopt::Long::config('auto_abbrev');


my $status;
my $TIMEOUT = 15;

my %ERRORS = ('UNKNOWN' , '-1',
              'OK' , '0',
              'WARNING', '1',
              'CRITICAL', '2');


my $state = "UNKNOWN";
my $answer = "";
my $snmpkey = 1;
my $community = "public";
my $port = 161;
my $filer = 0;
my @snmpoids;
my $hostname;
my $session;
my $error;
my $response;
my $critical = 0;		# number of critical events
my $warning = 0;		# number of warning events

my $snmpdfIndex = '1.3.6.1.4.1.789.1.5.4.1.1';
# The name for the referenced file system.
my $snmpdfFilesSys = '1.3.6.1.4.1.789.1.5.4.1.2';
# The total capacity in KBytes for the referenced file system.
my $snmpdfKBytesTotal = '1.3.6.1.4.1.789.1.5.4.1.3';
# The total disk space in KBytes that is in use on the referenced file system
my $snmpdfKBytesUsed = '1.3.6.1.4.1.789.1.5.4.1.4';
# The total disk space in KBytes that is in free for use on the referenced file system.
my $snmpdfKBytesAvail = '1.3.6.1.4.1.789.1.5.4.1.5';
# The percentage of disk space currently in use on the referenced file system.
my $snmpdfPerCentKBytesCapacity = '1.3.6.1.4.1.789.1.5.4.1.6';
# The total number of inodes in use on the referenced file system
my $snmpdfInodesUsed = '1.3.6.1.4.1.789.1.5.4.1.7';
# The total number of inodes that are available for use on the referenced file system
my $snmpdfInodesFree = '1.3.6.1.4.1.789.1.5.4.1.8';
# The percentage of disk space currently in use based on inode counts, on the referenced file system
my $snmpdfPerCentInodeCapacity = '1.3.6.1.4.1.789.1.5.4.1.9';
# The name of the file on which this file system is mounted
my $snmpdfMountedOn = '1.3.6.1.4.1.789.1.5.4.1.10';
# The count of the maximum number of files allowable on the referenced file system.
my $snmpdfMaxFilesAvail = '1.3.6.1.4.1.789.1.5.4.1.11';
# The count of the number of files currently in use on the referenced file system.
my $snmpdfMaxFilesUsed = '1.3.6.1.4.1.789.1.5.4.1.12';
# The largest value to which the filesysMaxfilesAvail parameter above can
# be increased by reconfiguration on the referenced file system.
my $snmpdfMaxFilesPossible = '1.3.6.1.4.1.789.1.5.4.1.13';


my $dfIndex;
my $dfFilesSys;
my $dfKBytesTotal;
my $dfKBytesUsed;
my $dfKBytesAvail;
my $dfPerCentKBytesCapacity;
my $dfInodesUsed;
my $dfInodesFree;
my $dfPerCentInodeCapacity;
my $dfMountedOn;
my $dfMaxFilesAvail;
my $dfMaxFilesUsed;
my $dfMaxFilesPossible;

# own vars for optimized ouput
my $dfFilesTotal;
my $dfPerCentFilesAvail;
my $dfPerCentFilesUsed;

# define limits
my $KBytesWarn = 5000;		# free vol space in MByte
my $KBytesCrit = 1000;		# free vol space in MByte
my $InodesWarn = 100000;
my $InodesCrit = 50000;
my $MaxFilesWarn = 10000;
my $MaxFilesCrit = 1000;

my $volname;


sub usage {
  printf "\nMissing arguments!\n";
  printf "\n";
  printf "Perl Check filervolume plugin for NetSaint\n";
  printf  "monitors volume parameters of a NetApp filer\n";
  printf "usage: \n";
  printf "check_filervolume.pl [-d] -c <READCOMMUNITY> [-p <PORT>] -k <KEY>";
  printf " <HOSTNAME>";
  printf "\nCopyright (C) 2000 Christoph Kron\n";
  printf "check_filervolume.pl comes with ABSOLUTELY NO WARRANTY\n";
  printf "This programm is licensed under the terms of the ";
  printf "GNU General Public License\n(check source code for details)\n";
  printf "\n\n";
  exit $ERRORS{"UNKNOWN"};
}

# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub {
     print ("ERROR: No snmp response from $hostname (alarm)\n");
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);

my $hrefstart;
my $hrefend;

$status = GetOptions( "community=s",\$community,
                     "port=i",\$port,
                     "key=i",\$snmpkey);
if ($status == 0)
{
        &usage;
}
  
   #shift;
   $hostname  = shift || &usage;

   ($session, $error) = Net::SNMP->session(
      -hostname  => $hostname,
      -community => $community,
      -port      => $port
   );

   if (!defined($session)) {
      $state='UNKNOWN';
      $answer=$error;
      print ("$state: $answer");
      exit $ERRORS{$state};
   }

   $snmpdfIndex .= "." . $snmpkey;
   $snmpdfFilesSys .= "." . $snmpkey;
   $snmpdfKBytesTotal .= "." . $snmpkey;
   $snmpdfKBytesUsed .= "." . $snmpkey;
   $snmpdfKBytesAvail .= "." . $snmpkey;
   $snmpdfPerCentKBytesCapacity .= "." . $snmpkey;
   $snmpdfInodesUsed .= "." . $snmpkey;
   $snmpdfInodesFree .= "." . $snmpkey;
   $snmpdfPerCentInodeCapacity .= "." . $snmpkey;
   $snmpdfMountedOn .= "." . $snmpkey;
   $snmpdfMaxFilesAvail .= "." . $snmpkey;
   $snmpdfMaxFilesUsed .= "." . $snmpkey;
   $snmpdfMaxFilesPossible .= "." . $snmpkey;

   push(@snmpoids,$snmpdfIndex);
   push(@snmpoids,$snmpdfFilesSys);
   push(@snmpoids,$snmpdfKBytesTotal);
   push(@snmpoids,$snmpdfKBytesUsed);
   push(@snmpoids,$snmpdfKBytesAvail);
   push(@snmpoids,$snmpdfPerCentKBytesCapacity);
   push(@snmpoids,$snmpdfInodesUsed);
   push(@snmpoids,$snmpdfInodesFree);
   push(@snmpoids,$snmpdfPerCentInodeCapacity);
   push(@snmpoids,$snmpdfMountedOn);
   push(@snmpoids,$snmpdfMaxFilesAvail);
   push(@snmpoids,$snmpdfMaxFilesUsed);
   push(@snmpoids,$snmpdfMaxFilesPossible);

   if (!defined($response = $session->get_request(@snmpoids))) {
      $answer=$session->error;
      $session->close;
      $state = 'UNKNOWN';
      print ("$answer,$community,$snmpkey");
      exit $ERRORS{$state};
   }

   $dfIndex = $response->{$snmpdfIndex};
   $dfFilesSys = $response->{$snmpdfFilesSys};
   $dfKBytesTotal = $response->{$snmpdfKBytesTotal};
   $dfKBytesUsed = $response->{$snmpdfKBytesUsed};
   $dfKBytesAvail = $response->{$snmpdfKBytesAvail};
   $dfPerCentKBytesCapacity = $response->{$snmpdfPerCentKBytesCapacity};
   $dfInodesUsed = $response->{$snmpdfInodesUsed};
   $dfInodesFree = $response->{$snmpdfInodesFree};
   $dfPerCentInodeCapacity = $response->{$snmpdfPerCentInodeCapacity};
   $dfMountedOn = $response->{$snmpdfMountedOn};
   $dfMaxFilesAvail = $response->{$snmpdfMaxFilesAvail};
   $dfMaxFilesUsed = $response->{$snmpdfMaxFilesUsed};
   $dfMaxFilesPossible = $response->{$snmpdfMaxFilesPossible};

   # close SNMP Session
   $session->close;


   # check limits


   # compare MegaBytes Values
   if ($dfKBytesAvail/1024 < $KBytesCrit) {
      $critical++;
   }
   elsif ($dfKBytesAvail/1024 < $KBytesWarn) {
         $warning++;                  
   }

   if ($dfInodesFree < $InodesCrit) {
      $critical++;
   }
   elsif ($dfInodesFree < $InodesWarn) {
         $warning++;                  
   }

   if ($dfMaxFilesAvail < $MaxFilesCrit) {
      $critical++;
   }
   elsif ($dfMaxFilesAvail < $MaxFilesWarn) {
         $warning++;                  
   }

   if ($critical > 0) {
      $state='CRITICAL';
   }
   elsif ($warning > 0) {
      $state='WARNING';
   }
   else {
      $state = 'OK';
   }

  $volname = $dfFilesSys;
  $volname =~ s/\//-/g;
  # remove . 
  $volname=~ s/\.//g;
  # remove last - from root volume
  $volname=~ s/-$//g;

  $hrefstart = "";
  $hrefend = "";

  $answer = sprintf("%s%s(%1.1f) volusage: %1.0f MB(%d%%) used, %1.0fMB (%d%%) avail %1.0f MB total",
		    $hrefstart,
                    $dfFilesSys,
		    $dfIndex,
		    $dfKBytesUsed / 1024,
		    $dfPerCentKBytesCapacity,
		    $dfKBytesAvail / 1024,
		    100 - $dfPerCentKBytesCapacity,
		    $dfKBytesTotal/1024); 

  $answer .= sprintf("<BR>inodes: %1.0f(%d%%) used, %1.0f(%d%%) avail<BR>",
		    $dfInodesUsed,
		    $dfPerCentInodeCapacity,
		    $dfInodesFree,
		    100 - $dfPerCentInodeCapacity);


  # calcalute percentage values
  $dfFilesTotal =  $dfMaxFilesAvail + $dfMaxFilesUsed;
  $dfPerCentFilesAvail = 100 / $dfFilesTotal * $dfMaxFilesAvail;
  $dfPerCentFilesUsed = 100 - $dfPerCentFilesAvail;

  $answer .= sprintf("maxfiles: %1.0f(%d%%) used, %1.0f(%d%%) avail %1.0f possible",
		    $dfMaxFilesUsed,
		    $dfPerCentFilesUsed,
		    $dfMaxFilesAvail,
		    $dfPerCentFilesAvail,
		    $dfMaxFilesPossible);

  $answer .= sprintf("<BR>mountpoint: %s%s\n",
  		    $dfMountedOn,
		    $hrefend);

print ("$answer");
exit $ERRORS{$state};
